"""
Computes the correlation between two contact
probabilities maps
"""
import numpy as np
import matplotlib
import matplotlib.pyplot as plt
from scipy.stats import pearsonr, spearmanr

from utils.paths import ANALYSIS_PATH, THERMO_PATH

# graphical settings
matplotlib.rcParams['font.size'] = 20
matplotlib.rcParams['text.usetex'] = True
matplotlib.rcParams['font.family'] = 'modern computer'

######################################################
# title
prot = "1srl"
title = "Comparison contact maps probabilities for different LJ choices (SH3)"

# Labels and files
x_prob = "LJ_126"
# x_input_path = ANALYSIS_PATH / prot /  "probability_IT_29Oct22_CV_2Nov22_CV.npy"
x_input_path = THERMO_PATH / "1srl_TSE_probability_cm_LJ12-6_3Nov22.npy"

y_prob = "LJ_1210"
# y_input_path = ANALYSIS_PATH / prot /  "probability_IE-_29Oct22_CV_2Nov22_CV.npy"
y_input_path = THERMO_PATH /  "1srl_TSE_probability_cm_LJ12-10_6Nov22.npy"
######################################################

# Load
x_cm = np.load(x_input_path)
y_cm = np.load(y_input_path)

# Select only those contacts which are in common between
# the two maps
y_cm_reduce = np.copy(y_cm)
y_cm_reduce[x_cm == 0] = 0
x_cm_reduce = np.copy(x_cm)
x_cm_reduce[y_cm == 0] = 0

x_cm_flat = x_cm_reduce.flatten()
y_cm_flat = y_cm_reduce.flatten()

# remove zeros
x_nonzero = x_cm_flat[x_cm_flat!=0]
y_nonzero = y_cm_flat[y_cm_flat!=0]

assert np.all((x_nonzero!=0) == (y_nonzero!=0)), print((x_nonzero!=0).sum(), (y_nonzero!=0).sum())

# # computing p - <p>
# x_cm_flat_contacts = x_cm_flat[x_cm_flat!=0].copy()
# y_cm_flat_contacts = y_cm_flat[y_cm_flat!=0].copy()
# x_p_diff = x_cm_flat_contacts - x_cm_flat_contacts.mean()
# y_p_diff = y_cm_flat_contacts - y_cm_flat_contacts.mean()

# Correlation
# compute Pearson's r
r, p_r = pearsonr(x_nonzero, y_nonzero)
# compute Spearman's r
r_s, p_s = spearmanr(x_nonzero, y_nonzero)
s = f"""
Correlation\n
---------------------------------------------\n
Pearson's r = {r:.3f}; p-value = {p_r:.3E}\n
Spearman's r = {r_s:.3f}; p-value = {p_s:.3E}\n
"""
print(s)

# Plot
fig, ax = plt.subplots(1, figsize=(9,9))
# plt.plot(x_p_diff, y_p_diff, 'o', alpha=0.5)
ax.plot(x_nonzero, y_nonzero, 'o', alpha=0.5)
ax.plot([0,1], [0, 1], '-', color='red', alpha=0.6)

ax.grid(alpha = 0.4)
ax.set_xlim(-0.03, 1.03)
ax.set_ylim(-0.03, 1.03)

ax.set_xlabel(x_prob)
ax.set_ylabel(y_prob)
ax.set_title(title)
fig.savefig(
    ANALYSIS_PATH / prot / "img" / f"correlation_contact_maps_{x_prob}_{y_prob}.pdf",
    bbox_inches='tight'
)
