'''Computation of the distribution of time needed
for each type of trajectory to reach the native state'''
import numpy as np
import matplotlib
import matplotlib.pyplot as plt
import seaborn as sns
import pandas as pd

from utils.misc import block_avg
from utils.paths import ANALYSIS_PATH

# graphical settings
sns.set_theme(style="ticks")
matplotlib.rcParams['font.size'] = 24
matplotlib.rcParams['text.usetex'] = True
matplotlib.rcParams['font.family'] = 'modern computer'

##########################
prot_name = '1ucs'
dataset = '29Oct22_CV_2Nov22_CV'
PROT_PATH = ANALYSIS_PATH / prot_name

# loading timeseries
Q = np.load(PROT_PATH / f'Q_{dataset}.npy')
folded_trajectories = np.load(PROT_PATH / f'folded_trajectories_{dataset}.npy')
id_map = np.loadtxt(PROT_PATH / f"ID_mapping_{dataset}.csv", delimiter=",", dtype=int)

# Load pathway classification
THR = np.load(PROT_PATH / f'THR_traj_{dataset}.npy')
BT = np.load(PROT_PATH / f'BT_traj_{dataset}.npy')

# Find WHEN a particular trajectory is folded
Q_folded = 0.8
block_size = 250
t_fold = pd.DataFrame(
    {
        't': [None]*200,
        'type': [None]*200
    }
)
globalindex = 0

for q, id_tuple in zip(Q, id_map):
    # Q is used instead of Q[folded_trajectories] because indexes
    # related to BT and THR are related the the formers

    # A block average is performed to avg out fluctuations
    q_block, _ = block_avg(q, block_size)
    idx_folded = np.where((q_block>Q_folded)==True)[0]

    if idx_folded.size != 0:
        if id_tuple[1] in BT:
            t_fold['type'][globalindex] = 'BT'
            t_fold['t'][globalindex] = int(idx_folded[0]*block_size)
            globalindex += 1
        elif id_tuple[1] in THR:
            t_fold['type'][globalindex] = 'THR'
            t_fold['t'][globalindex] = int(idx_folded[0]*block_size)
            globalindex += 1
        else:
            if id_tuple[1] not in folded_trajectories:
                print(f"Check run {id_tuple[1]}")
            t_fold['type'][globalindex] = 'IE-'
            t_fold['t'][globalindex] = int(idx_folded[0]*block_size)
            globalindex += 1

len_sim = Q[0].shape[0]
##########################
dataset2 = '29Oct22_CV_increment_2Nov22_CV_increment'

# loading timeseries
Q = np.load(PROT_PATH / f'Q_{dataset2}.npy')
folded_trajectories = np.load(PROT_PATH / f'folded_trajectories_{dataset2}.npy')
id_map = np.loadtxt(PROT_PATH / f"ID_mapping_{dataset2}.csv", delimiter=",", dtype=int)

# Load pathway classification
THR = np.load(PROT_PATH / f'THR_traj_{dataset2}.npy')
BT = np.load(PROT_PATH / f'BT_traj_{dataset2}.npy')

for q, id_tuple in zip(Q, id_map):
    # Q is used instead of Q[folded_trajectories] because indexes
    # related to BT and THR are related the the formers

    # A block average is performed to avg out fluctuations
    q_block, _ = block_avg(q, block_size)
    idx_folded = np.where((q_block>Q_folded)==True)[0]

    if idx_folded.size != 0:
        if id_tuple[1] in BT:
            t_fold['type'][globalindex] = 'BT'
            t_fold['t'][globalindex] = int(idx_folded[0]*block_size) + len_sim
            globalindex += 1
        elif id_tuple[1] in THR:
            t_fold['type'][globalindex] = 'THR'
            t_fold['t'][globalindex] = int(idx_folded[0]*block_size) + len_sim
            globalindex += 1
        else:
            if id_tuple[1] == 8:
                print(f"This {id_tuple[1]} was misclassified, hence is present in the Q and the id_map arrays but has been already classified in the non-incremented trajectories")
            else:
                print(f"Run misclassified: {id_tuple[1]}")

##########################
# including SH3
prot_name2 = '1srl'
dataset3 = '31Oct22_7Nov22'
PROT_PATH2 = ANALYSIS_PATH / prot_name2

# loading timeseries
Q2 = np.load(PROT_PATH2 / f'Q_{dataset3}.npy', allow_pickle=True)
folded_trajectories2 = np.load(PROT_PATH2 / f'folded_trajectories_{dataset3}.npy')
id_map2 = np.loadtxt(PROT_PATH2 / f"ID_mapping_{dataset3}.csv", delimiter=",", dtype=int)

Q_folded2 = 0.7
block_size = 250

for q, id_tuple in zip(Q2, id_map2):
    q_block, _ = block_avg(q, block_size)
    idx_folded = np.where((q_block>Q_folded2)==True)[0]
    if idx_folded.size != 0:
        t_fold['type'][globalindex] = 'SH3'
        t_fold['t'][globalindex] = int(idx_folded[0]*block_size)
        globalindex += 1

# including extention of the 7Nov22
dataset4 = '7Nov22_increment'
# loading timeseries
Q4 = np.load(PROT_PATH2 / f'Q_{dataset4}.npy', allow_pickle=True)
folded_trajectories4 = np.load(PROT_PATH2 / f'folded_trajectories_{dataset4}.npy')
id_map4 = np.loadtxt(PROT_PATH2 / f"ID_mapping_{dataset4}.csv", delimiter=",", dtype=int)

for q, id_tuple in zip(Q4, id_map4):
    q_block, _ = block_avg(q, block_size)
    idx_folded = np.where((q_block>Q_folded2)==True)[0]
    if idx_folded.size != 0:
        t_fold['type'][globalindex] = 'SH3'
        t_fold['t'][globalindex] = int(idx_folded[0]*block_size)
        globalindex += 1

# remove empty trajectories
t_fold = t_fold.dropna()

# checks
print("TOT: ", globalindex, "\n")
print("FOL: ", len(t_fold[t_fold['type'] == "IE-"]), "\n")
print("THR: ", len(t_fold[t_fold['type'] == "THR"]), "\n")
print("BT: ", len(t_fold[t_fold['type'] == "BT"]), "\n")
print("SH3: ", len(t_fold[t_fold['type'] == "SH3"]), "\n")

# mean folding time
print("Mean FOL: ", t_fold[t_fold['type'] == "IE-"]["t"].mean())
print("Mean THR: ", t_fold[t_fold['type'] == "THR"]["t"].mean())
print("Mean BT: ", t_fold[t_fold['type'] == "BT"]["t"].mean())
print("Mean SH3: ", t_fold[t_fold['type'] == "SH3"]["t"].mean())

# Bar plot
f, ax = plt.subplots(figsize=(7, 6))
# Plot the orbital period with horizontal boxes
sns.boxplot(x="t", y="type", data=t_fold,
            width=.6, palette="vlag",
            fliersize=0) # remove icon for outliers
# Add in points to show each observation
sns.stripplot(x="t", y="type", data=t_fold,
              size=3, color=".3", linewidth=0)

ax.set(
    ylabel="",
    xlabel='Time [MD steps]'
)
ax.xaxis.grid(True, which='both', alpha=0.4)
ax.xaxis.set_minor_locator(plt.MultipleLocator(0.5e4))
ax.xaxis.set_major_locator(plt.MultipleLocator(1e4))
ax.set_xlim(0, 9e4)
ax.ticklabel_format(axis="x", scilimits=(0,4))
sns.despine(trim=True, left=True)

ax.tick_params(
    axis='y',           # changes apply to the x-axis
    which='major',
    left=False,         # ticks along the bottom edge are off
)

# final annotation
ax.annotate('RD1', xy=(-0.15, 0.625), xytext=(-0.25, 0.625),
            fontsize=12, ha='center', va='center', xycoords='axes fraction',
            arrowprops=dict(arrowstyle='-[, widthB=7.5, lengthB=.5', lw=1.0, color='k'))

f.savefig(PROT_PATH / 'img' / f'time_folding_{dataset}_{dataset2}_1srl_{dataset3}.pdf', bbox_inches='tight')