"""
Select those natively entangled contacts found in the IE-
ensemble to be highlighted in the t* vs k plot.

This shows consistency in the choice of the reaction coordinate,
because two separate analysis shows which contacts (in a specific subset)
form first.
"""
import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1.axes_divider import make_axes_locatable

from utils.paths import ANALYSIS_PATH

ANALYSIS_PATH = ANALYSIS_PATH / "1ucs"

# load probability maps
ie = np.load(ANALYSIS_PATH / "probability_IE-_29Oct22_CV_2Nov22_CV.npy")
ie_c = ie.copy()

# Statistics for Z-score
ie_mean = np.mean(ie[ie!=0])
ie_std = np.std(ie[ie!=0])
ie_c[ie_c!=0] = (ie_c[ie_c!=0] - ie_mean)/ie_std

# plots and prints
fig, axs = plt.subplots(1, 2, figsize=(10,8))
for ax in axs.flatten():
    ax.grid(alpha=0.5)

h = axs[0].imshow(ie_c, cmap="binary")
ax0_div = make_axes_locatable(axs[0])
cax1 = ax0_div.append_axes("right", size="7%", pad="2%")
fig.colorbar(h, cax=cax1)

ie_c_bin = ie_c > 0
axs[1].imshow(ie_c_bin, cmap="binary")

plt.show()

arr1, arr2 = np.nonzero(ie_c_bin)
out = [(i1, i2) for i1, i2 in zip(arr1, arr2)]
print(out)
