import numpy as np

cimport cython
from libc.math cimport sqrt

DTYPE = np.double

@cython.boundscheck(False)  # Deactivate bounds checking
@cython.wraparound(False)   # Deactivate negative indexing.
cdef double distance(
    double[:, :] pos_ts,
    Py_ssize_t atom1,
    Py_ssize_t atom2):

    cdef double dx, dy, dz

    dx = pos_ts[atom1,0] - pos_ts[atom2,0]
    dy = pos_ts[atom1,1] - pos_ts[atom2,1]
    dz = pos_ts[atom1,2] - pos_ts[atom2,2]
    return sqrt(dx*dx + dy*dy + dz*dz)

@cython.boundscheck(False) 
@cython.wraparound(False)
@cython.cdivision(True) # avoide checking the 0 division
def q_time_series(
    u,
    double total_contacts,
    double[:, :] cm_mod,
    double gamma):

    cdef Py_ssize_t n = cm_mod.shape[0]
    cdef Py_ssize_t frames = u.trajectory.n_frames

    cdef double[:] q = np.zeros(frames, dtype=DTYPE)
    cdef double[:,:] pos_ts

    cdef Py_ssize_t i, j, k
    cdef double tot

    for i in range(frames):
        pos_ts = (u.trajectory[i].positions).astype(DTYPE)

        tot = 0
        for j in range(n):
            for k in range(j+1, n):
                if distance(pos_ts, j, k) <= gamma*cm_mod[j,k]:
                    tot += 1
        
        q[i] = tot/total_contacts
    
    return q
